function images = pco_adaptor_example_softwaretrigger(...
    imcount_per_trigger, trigger_count, timeout_s)
% This function records and returns a series of images using the software 
% triggering of the imaq adaptor. Therefore a number of images per trigger
% is specified as well as a number of required triggers, the overall number
% of images will then be the product(imcount_per_trigger * trigger_count).
% The software trigger is forced by a keypress of the user.
%
%   [images] = pco_adaptor_example_softwaretrigger(...
%       imcount_per_trigger, trigger_count, timeout_s)
%
% * Input parameters
%       imcount_per_trigger:    Number of images per soft trigger
%       trigger_count:          Number of triggers 
%       timeout_s:              Number of seconds to wait before video 
%                               input timeout occurs
%
% * Output parameters
%       images:                 Stack with acquired images
%
% - Set property values and trigger mode
% - Start preview, acquire images by soft-trigger(keypress), stop preview
% - Display images and reset the adaptor
%

%% Set default values for paramters that haven't been set
if(~exist('imcount_per_trigger','var'))
    imcount_per_trigger = 4;
end

if(~exist('trigger_count','var'))
    trigger_count = 3;
end

if(~exist('timeout_s','var'))
    timeout_s = 120;
end

%Identify required adaptor
if verLessThan('matlab','9.0') %older than R2016a
  error('This adaptor is supported in Matlab R2016a and later versions'); 
else %R2016a and newer
  if(strcmp(computer('arch'),'win64'))
    adaptorName = ['pcocameraadaptor_r' version('-release')];
  elseif(strcmp(computer('arch'),'glnxa64'))
    adaptorName = ['pcocameraadaptor_r' version('-release')];
  else
    error('This platform is not supported.');
  end
end

%% Configure camera and record images
%Create video input object
vid = videoinput(adaptorName, 0);

%Create adaptor source
src = getselectedsource(vid);

%Set logging to memory
vid.LoggingMode = 'memory';

%Set the timeout property of video input object 'vid'
set(vid,'Timeout',timeout_s);

%Set timestamp mode
src.TMTimestampMode = 'BinaryAndAscii';

%Set frames per trigger
vid.FramesPerTrigger = imcount_per_trigger;

%Set trigger mode to manual
triggerconfig(vid, 'manual');

% TriggerRepeat is zero based and is always 
% one less than the number of triggers.
vid.TriggerRepeat = trigger_count - 1;

%Start preview
preview(vid);

%Start acquisition
start(vid);

% Force the soft trigger by pressing any key
% For every soft trigger a defined number of images is recorded, extracted
% and added to the image series
for i = 0 : vid.TriggerRepeat
    %Wait for keypress
    pause;
    
    %Force trigger command
    trigger(vid);
    
    %Read out all images and remove them from memory buffer
    %Function blocks until the images have been acquired
    images_temp = getdata(vid,imcount_per_trigger);
    
    if i==0
        %For first image series assign it to image vector
        images = images_temp;
    else
        %Append new image series to vector
        images = cat(4,images,images_temp);
    end
end

%Stop preview
stoppreview(vid);

%% Show images
%Show images in Video Viewer app
implay(images);

%Reset adaptor
imaqreset;


